<?php
declare(strict_types=1);

use Migrations\AbstractMigration;

class Initial extends AbstractMigration {
    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Up Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-up-method
     * @return void
     */
    public function up(): void {
        if (!$this->hasTable('snmptt')) {
            $this->table('snmptt')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('eventname', 'string', [
                    'default' => null,
                    'limit'   => 50,
                ])
                ->addColumn('eventid', 'string', [
                    'default' => null,
                    'limit'   => 50,
                ])
                ->addColumn('trapoid', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('enterprise', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('community', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('hostname', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('agentip', 'string', [
                    'default' => null,
                    'limit'   => 16,
                ])
                ->addColumn('category', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('severity', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('uptime', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('traptime', 'string', [
                    'default' => null,
                    'limit'   => 30,
                ])
                ->addColumn('formatline', 'string', [
                    'default' => null,
                    'limit'   => 255,
                ])
                ->addColumn('trapread', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addIndex([
                    'hostname',
                    'severity'
                ], [
                    'name' => 'snmptt_host_sev'
                ])
                ->create();
        }

        if (!$this->hasTable('snmptt_archive')) {
            $this->table('snmptt_archive')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('snmptt_id', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('eventname', 'string', [
                    'default' => null,
                    'limit'   => 50,
                ])
                ->addColumn('eventid', 'string', [
                    'default' => null,
                    'limit'   => 50,
                ])
                ->addColumn('trapoid', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('enterprise', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('community', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('hostname', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('agentip', 'string', [
                    'default' => null,
                    'limit'   => 16,
                ])
                ->addColumn('category', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('severity', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('uptime', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('traptime', 'string', [
                    'default' => null,
                    'limit'   => 30,
                ])
                ->addColumn('formatline', 'string', [
                    'default' => null,
                    'limit'   => 255,
                ])
                ->addColumn('trapread', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->create();
        }

        if (!$this->hasTable('snmptt_jobs')) {
            $this->table('snmptt_jobs')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('type', 'string', [
                    'default' => null,
                    'limit'   => 50,
                ])
                ->addColumn('jobstate', 'integer', [
                    'default' => 0,
                    'limit'   => 5,
                    'null'    => false,
                ])
                ->addColumn('count', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('jobtime', 'integer', [
                    'default' => 0,
                    'limit'   => 50,
                    'null'    => false,
                ])
                ->addColumn('message', 'string', [
                    'default' => '',
                    'limit'   => 255,
                    'null'    => false,
                ])
                ->create();

            $this->table('snmptt_jobs')->insert([['type' => 'archive'], ['type' => 'delete']])->save();
        }

        if (!$this->hasTable('snmptt_statistics')) {
            $this->table('snmptt_statistics')
                ->addColumn('stat_time', 'string', [
                    'default' => null,
                    'limit'   => 30,
                ])
                ->addColumn('total_received', 'integer', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('total_translated', 'integer', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('total_ignored', 'integer', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('total_unknown', 'integer', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->create();
        }

        if (!$this->hasTable('snmptt_unknown')) {
            $this->table('snmptt_unknown')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('trapoid', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('enterprise', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('community', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('hostname', 'string', [
                    'default' => null,
                    'limit'   => 100,
                ])
                ->addColumn('agentip', 'string', [
                    'default' => null,
                    'limit'   => 16,
                ])
                ->addColumn('uptime', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('traptime', 'string', [
                    'default' => null,
                    'limit'   => 30,
                ])
                ->addColumn('formatline', 'string', [
                    'default' => null,
                    'limit'   => 255,
                ])
                ->addColumn('severity', 'string', [
                    'default' => null,
                    'limit'   => 20,
                ])
                ->addColumn('trapread', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addIndex([
                    'hostname',
                    'severity'
                ], [
                    'name' => 'snmptt_unknown_host_sev'
                ])
                ->create();
        }
    }

    /**
     * Down Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-down-method
     * @return void
     */
    public function down(): void {
        $this->table('snmptt_unknown')->drop()->save();
        $this->table('snmptt_statistics')->drop()->save();
        $this->table('snmptt_jobs')->drop()->save();
        $this->table('snmptt_archive')->drop()->save();
        $this->table('snmptt')->drop()->save();
    }
}
