<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use App\Model\Table\SystemsettingsTable;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\UUID;
use Migrations\AbstractSeed;

/**
 * Class InstallSeed
 *
 * Created:
 * oitc bake seed -p SnmpTrapModule --data Install
 *
 * Apply:
 * oitc migrations seed -p SnmpTrapModule
 */
class InstallSeed extends AbstractSeed {
    /**
     * Run Method.
     *
     * Write your database seeder using this method.
     *
     * More information on writing seeds is available here:
     * https://book.cakephp.org/migrations/3/en/index.html#seed-seeding-your-database
     *
     * @return void
     */
    public function run(): void {
        //Cronjobs
        $table = $this->table('cronjobs');

        $data = [
            [
                'task'     => 'SnmpTrapClearDatabase',
                'plugin'   => 'SnmpTrapModule',
                'interval' => '1440',
                'enabled'  => '1',
            ]
        ];

        //Check if records exists
        foreach ($data as $index => $record) {
            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($table->getName())
                ->where([
                    'plugin' => $record['plugin'],
                    'task'   => $record['task']
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $table->insert($record)->save();
            }
        }

        //Commands
        $table = $this->table('commands');

        $checkSnmptrapsCommandUUID = UUID::v4();
        $data = [
            [
                'name'         => 'check_snmptraps',
                'command_line' => '$USER1$/check_snmptraps.pl -H $HOSTADDRESS$ -w -c -m -O "$ARG1$"',
                'command_type' => CHECK_COMMAND,
                'human_args'   => null,
                'uuid'         => $checkSnmptrapsCommandUUID,
                'description'  => 'This plugin checks incoming snmp traps for warning and critical state.'
            ]
        ];

        //Check if records exists
        foreach ($data as $index => $record) {
            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($table->getName())
                ->where([
                    'command_type' => CHECK_COMMAND,
                    'name'         => $record['name']
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $table->insert($record)->save();
            }
        }

        $checkSnmptrapsCommandID = $QueryBuilder->select('id')
            ->where([
                'name' => 'check_snmptraps',
            ])
            ->execute()->fetch()[0];


        //Commandarguments
        $table = $this->table('commandarguments');

        $data = [
            [
                'command_id' => $checkSnmptrapsCommandID,
                'name'       => '$ARG1$',
                'human_name' => 'Match OID'
            ]
        ];

        //Check if records exists
        foreach ($data as $index => $record) {
            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($table->getName())
                ->where([
                    'command_id' => $checkSnmptrapsCommandID,
                    'name'       => $record['name']
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $table->insert($record)->save();
            }
        }

        $checkSnmptrapsCommandargumentID = $QueryBuilder->select('id')
            ->where([
                'command_id' => $checkSnmptrapsCommandID,
                'name'       => '$ARG1$'
            ])
            ->execute()->fetch()[0];


        //Servicetemplates
        $table = $this->table('servicetemplates');
        $ctsTable = $this->table('contacts_to_servicetemplates');
        $servicetemplateUUID = UUID::v4();

        $data = [
            [
                "name"                         => "Check SNMP Trap",
                "uuid"                         => $servicetemplateUUID,
                "template_name"                => "CHECK_SNMPTRAP",
                "servicetemplatetype_id"       => "1",
                "description"                  => "",
                "command_id"                   => $checkSnmptrapsCommandID,
                "check_command_args"           => "",
                "checkcommand_info"            => "",
                "check_interval"               => "60",
                "retry_interval"               => "60",
                "max_check_attempts"           => "1",
                "first_notification_delay"     => "0",
                "notification_interval"        => "7200",
                "notify_on_warning"            => "1",
                "notify_on_unknown"            => "1",
                "notify_on_critical"           => "1",
                "notify_on_recovery"           => "1",
                "notify_on_flapping"           => "0",
                "notify_on_downtime"           => "0",
                "flap_detection_enabled"       => "0",
                "flap_detection_on_ok"         => "0",
                "flap_detection_on_warning"    => "0",
                "flap_detection_on_unknown"    => "0",
                "flap_detection_on_critical"   => "0",
                "low_flap_threshold"           => "0",
                "high_flap_threshold"          => "0",
                "process_performance_data"     => "1",
                "freshness_checks_enabled"     => "0",
                "freshness_threshold"          => 3600,
                "passive_checks_enabled"       => "1",
                "event_handler_enabled"        => "0",
                "active_checks_enabled"        => "1",
                "retain_status_information"    => "0",
                "retain_nonstatus_information" => "0",
                "notifications_enabled"        => "1",
                "notes"                        => "",
                "priority"                     => "1",
                "tags"                         => "",
                "service_url"                  => "",
                "is_volatile"                  => "1",
                "check_freshness"              => "0",
                "container_id"                 => 1,
                "eventhandler_command_id"      => 0,
                "check_period_id"              => 1,
                "notify_period_id"             => 1
            ]
        ];

        //Check if records exists
        foreach ($data as $index => $record) {
            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($table->getName())
                ->where([
                    'servicetemplatetype_id' => "1",
                    'template_name'          => 'CHECK_SNMPTRAP',
                ])
                ->execute();
            $result = $stm->fetchAll();
            if (!empty($result)) {
                $checkSnmptrapsServicetemplateID = $QueryBuilder->select('id')
                    ->where([
                        'servicetemplatetype_id' => "1",
                        'template_name'          => 'CHECK_SNMPTRAP',
                    ])
                    ->execute()->fetch()[0];
            } else {
                $table->insert($record)->save();

                $checkSnmptrapsServicetemplateID = $QueryBuilder->select('id')
                    ->where([
                        'uuid' => $record['uuid'],
                    ])
                    ->execute()->fetch()[0];
                $ctsTable->insert([
                    'contact_id'         => 1,
                    'servicetemplate_id' => $checkSnmptrapsServicetemplateID
                ])->save();
            }
        }

        //Servicetemplatecommandargumentvalues
        $table = $this->table('servicetemplatecommandargumentvalues');

        $data = [
            [
                'commandargument_id' => $checkSnmptrapsCommandargumentID,
                'servicetemplate_id' => $checkSnmptrapsServicetemplateID,
                'value'              => '',
            ]
        ];

        //Check if records exists
        foreach ($data as $index => $record) {
            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($table->getName())
                ->where([
                    'commandargument_id' => $checkSnmptrapsCommandargumentID,
                    'servicetemplate_id' => $checkSnmptrapsServicetemplateID
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $table->insert($record)->save();
            }
        }


        /** @var SystemsettingsTable $SystemsettingsTable */
        $SystemsettingsTable = TableRegistry::getTableLocator()->get('Systemsettings');
        if (!$SystemsettingsTable->existsByKey('INIT.SNMPTRAPD_RESTART')) {
            $SNMPTRAPD_RESTART = $SystemsettingsTable->newEntity([
                'section' => 'INIT',
                'key'     => 'INIT.SNMPTRAPD_RESTART',
                'value'   => 'systemctl restart snmptrapd.service',
                'info'    => 'Command to restart the snmptrapd service required by the SnmpTrapModule'
            ]);
            $SystemsettingsTable->save($SNMPTRAPD_RESTART);
        }
        if (!$SystemsettingsTable->existsByKey('INIT.SNMPTT_RESTART')) {
            $SNMPTT_RESTART = $SystemsettingsTable->newEntity([
                'section' => 'INIT',
                'key'     => 'INIT.SNMPTT_RESTART',
                'value'   => 'systemctl restart snmptt.service',
                'info'    => 'Command to restart the snmptt service required by the SnmpTrapModule'
            ]);
            $SystemsettingsTable->save($SNMPTT_RESTART);
        }
    }
}
