<?php
// Copyright (C) <2020>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//    If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//    under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//    License agreement and license key will be shipped with the order
//    confirmation.

namespace SnmpTrapModule\Controller;


use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use SnmpTrapModule\Filter\SnmpttListIndexFilter;
use SnmpTrapModule\Model\Table\SnmpttTable;

class SnmpttListController extends AppController {

    public function index() {
        if (!$this->isAngularJsRequest()) {
            //Only ship template
            return;
        }

        /** @var SnmpttTable $SnmpttTable */
        $SnmpttTable = TableRegistry::getTableLocator()->get('SnmpTrapModule.Snmptt');

        $SnmpttListIndexFilter = new SnmpttListIndexFilter($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $SnmpttListIndexFilter->getPage());

        $snmptt_entries = $SnmpttTable->getSnmpttIndex($SnmpttListIndexFilter, $PaginateOMat);

        $this->set('snmptt_entries', $snmptt_entries);
        $toJson = ['snmptt_entries', 'paging'];
        if ($this->isScrollRequest()) {
            $toJson = ['snmptt_entries', 'scroll'];
        }
        $this->viewBuilder()->setOption('serialize', $toJson);
    }

    public function delete($id) {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        /** @var SnmpttTable $SnmpttTable */
        $SnmpttTable = TableRegistry::getTableLocator()->get('SnmpTrapModule.Snmptt');

        if (!$SnmpttTable->existsById($id)) {
            throw new NotFoundException(__('Snmptt entry not found'));
        }

        $Snmptt = $SnmpttTable->get($id);

        if ($SnmpttTable->delete($Snmptt)) {
            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->response = $this->response->withStatus(500);
        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }
}