<?php
// Copyright (C) <2020>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//    If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//    under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//    License agreement and license key will be shipped with the order
//    confirmation.


namespace SnmpTrapModule\Lib\ConfigGenerator;


use itnovum\openITCOCKPIT\ConfigGenerator\ConfigGenerator;
use itnovum\openITCOCKPIT\ConfigGenerator\ConfigInterface;
use itnovum\openITCOCKPIT\ConfigGenerator\ConfigSymlink;
use itnovum\openITCOCKPIT\ConfigGenerator\FileHeader;
use Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;
use Twig\Loader\FilesystemLoader;

class SnmpTrapCfgs_snmptrapd extends ConfigGenerator implements ConfigInterface {

    protected $basePath = PLUGIN . 'SnmpTrapModule' . DS . 'src' . DS;

    protected $templateDir = 'config_templates';

    protected $template = 'snmptrapd.tpl';

    protected $realOutfile = '/etc/default/snmptrapd';

    protected $linkedOutfile = '/opt/openitc/etc/SnmpTrapModule/snmptrapd';

    /**
     * @var string
     */
    protected $commentChar = '#';

    protected $defaults = [
        'string' => [
            'trapd_enabled'  => 'yes',
            'snmptrapd_conf' => '/etc/snmp/snmptrapd.conf',
            'snmptrapd_pid'  => '/var/run/snmptrapd.pid'
        ]
    ];

    protected $dbKey = 'SnmpTrapCfgs_snmptrapd';

    /**
     * @param array $data
     * @return array|bool|true
     */
    public function customValidationRules($data) {
        return true;
    }

    /**
     * @return string
     */
    public function getAngularDirective() {
        return 'snmp-trap-cfgs-snmptrapd';
    }

    /**
     * @param string $key
     * @return string
     */
    public function getHelpText($key) {
        $help = [
            'trapd_enabled'  => 'Enable or disable the snmptrap daemon (yes/no)',
            'snmptrapd_conf' => 'Path to the snmptrap daemon configuration file',
            'snmptrapd_pid'  => 'Path to the snmptrap daemon pid file'
        ];

        if (isset($help[$key])) {
            return $help[$key];
        }

        return '';
    }

    /**
     * Save the configuration as text file on disk
     *
     * @param array $dbRecords
     * @return bool|int
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function writeToFile($dbRecords) {
        $config = $this->mergeDbResultWithDefaultConfiguration($dbRecords);
        $configToExport = [];
        foreach ($config as $type => $fields) {
            foreach ($fields as $key => $value) {
                $configToExport[$key] = $value;
            }
        }

        $success = true;

        $FileHeader = new FileHeader();
        $configToExport['STATIC_FILE_HEADER'] = $FileHeader->getHeader($this->commentChar);

        /*
         * Write:
         * - config.yml
         */
        $loader = new FilesystemLoader([
            $this->getTemplatePath()
        ]);
        $twig = new Environment($loader, ['debug' => true]);

        // /etc/default/snmptrapd
        $ConfigSymlink = new ConfigSymlink($this->realOutfile, $this->linkedOutfile);
        if (!file_put_contents($this->realOutfile, $twig->render($this->getTemplateName(), $configToExport))) {
            $success = false;
        }
        $ConfigSymlink->link();


        return $success;

    }

    /**
     * @param array $dbRecords
     * @return bool|array
     */
    public function migrate($dbRecords) {
        return $this->mergeDbResultWithDefaultConfiguration($dbRecords);
    }

}
