<?php
// Copyright (C) <2020>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//    If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//    under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//    License agreement and license key will be shipped with the order
//    confirmation.


namespace SnmpTrapModule\Lib\ConfigGenerator;


use itnovum\openITCOCKPIT\ConfigGenerator\ConfigGenerator;
use itnovum\openITCOCKPIT\ConfigGenerator\ConfigInterface;
use itnovum\openITCOCKPIT\ConfigGenerator\ConfigSymlink;
use itnovum\openITCOCKPIT\ConfigGenerator\FileHeader;
use Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;
use Twig\Loader\FilesystemLoader;

class SnmpTrapCfgs_snmpttIni extends ConfigGenerator implements ConfigInterface {

    protected $basePath = PLUGIN . 'SnmpTrapModule' . DS . 'src' . DS;

    protected $templateDir = 'config_templates';

    protected $template = 'snmptt.ini.tpl';

    protected $realOutfile = '/etc/snmp/snmptt.ini';

    protected $linkedOutfile = '/opt/openitc/etc/SnmpTrapModule/snmptt.ini';

    /**
     * @var string
     */
    protected $commentChar = '#';

    protected $defaults = [
        'int'    => [
            'unknown_trap_log_enable' => 0,
            'debugging'               => 0
        ],
        'string' => [
            'debugging_file' => '/var/log/snmptt/snmptt.debug',
        ]
    ];

    protected $dbKey = 'SnmpTrapCfgs_snmpttIni';

    /**
     * @param array $data
     * @return array|bool|true
     */
    public function customValidationRules($data) {
        return true;
    }

    /**
     * @return string
     */
    public function getAngularDirective() {
        return 'snmp-trap-cfgs-snmptt-ini';
    }

    /**
     * @param string $key
     * @return string
     */
    public function getHelpText($key) {
        $help = [
            'unknown_trap_log_enable' => 'Log SNMP traps without state in database (1/0)',
            'debugging'               => 'Write debug log (1/0)',
            'debugging_file'          => 'File path to save the debug log',
        ];

        if (isset($help[$key])) {
            return $help[$key];
        }

        return '';
    }

    /**
     * Save the configuration as text file on disk
     *
     * @param array $dbRecords
     * @return bool|int
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function writeToFile($dbRecords) {
        $config = $this->mergeDbResultWithDefaultConfiguration($dbRecords);
        $configToExport = [];
        foreach ($config as $type => $fields) {
            foreach ($fields as $key => $value) {
                $configToExport[$key] = $value;
            }
        }

        $success = true;

        $FileHeader = new FileHeader();
        $configToExport['STATIC_FILE_HEADER'] = $FileHeader->getHeader($this->commentChar);

        $mcp = new \App\itnovum\openITCOCKPIT\Database\MysqlConfigFileParser();
        $ini_file = $mcp->parse_mysql_cnf('/opt/openitc/etc/mysql/mysql.cnf');
        $configToExport['db_name'] = $ini_file['database'] ? $ini_file['database'] : 'openitcockpit';
        $configToExport['db_host'] = $ini_file['host'] ? $ini_file['host'] : 'localhost';
        $configToExport['db_port'] = $ini_file['port'] ? $ini_file['port'] : 3306;
        $configToExport['db_user'] = $ini_file['user'];
        $configToExport['db_password'] = $ini_file['password'];

        /*
         * Write:
         * - config.yml
         */
        $loader = new FilesystemLoader([
            $this->getTemplatePath()
        ]);
        $twig = new Environment($loader, ['debug' => true]);

        // /etc/default/snmptrapd
        $ConfigSymlink = new ConfigSymlink($this->realOutfile, $this->linkedOutfile);
        if (!file_put_contents($this->realOutfile, $twig->render($this->getTemplateName(), $configToExport))) {
            $success = false;
        }
        $ConfigSymlink->link();


        return $success;

    }

    /**
     * @param array $dbRecords
     * @return bool|array
     */
    public function migrate($dbRecords) {
        return $this->mergeDbResultWithDefaultConfiguration($dbRecords);
    }

}
