<?php
declare(strict_types=1);

namespace SnmpTrapModule\Model\Table;

use App\Lib\Traits\Cake2ResultTableTrait;
use App\Lib\Traits\PaginationAndScrollIndexTrait;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use SnmpTrapModule\Filter\SnmpttListIndexFilter;

/**
 * Snmptt Model
 *
 * @method \SnmpTrapModule\Model\Entity\Snmptt newEmptyEntity()
 * @method \SnmpTrapModule\Model\Entity\Snmptt newEntity(array $data, array $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt[] newEntities(array $data, array $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt get($primaryKey, $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \SnmpTrapModule\Model\Entity\Snmptt[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class SnmpttTable extends Table {

    use Cake2ResultTableTrait;
    use PaginationAndScrollIndexTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('snmptt');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('eventname')
            ->maxLength('eventname', 50)
            ->requirePresence('eventname', 'create')
            ->notEmptyString('eventname');

        $validator
            ->scalar('eventid')
            ->maxLength('eventid', 50)
            ->requirePresence('eventid', 'create')
            ->notEmptyString('eventid');

        $validator
            ->scalar('trapoid')
            ->maxLength('trapoid', 100)
            ->requirePresence('trapoid', 'create')
            ->notEmptyString('trapoid');

        $validator
            ->scalar('enterprise')
            ->maxLength('enterprise', 100)
            ->requirePresence('enterprise', 'create')
            ->notEmptyString('enterprise');

        $validator
            ->scalar('community')
            ->maxLength('community', 20)
            ->requirePresence('community', 'create')
            ->notEmptyString('community');

        $validator
            ->scalar('hostname')
            ->maxLength('hostname', 100)
            ->requirePresence('hostname', 'create')
            ->notEmptyString('hostname');

        $validator
            ->scalar('agentip')
            ->maxLength('agentip', 16)
            ->requirePresence('agentip', 'create')
            ->notEmptyString('agentip');

        $validator
            ->scalar('category')
            ->maxLength('category', 20)
            ->requirePresence('category', 'create')
            ->notEmptyString('category');

        $validator
            ->scalar('severity')
            ->maxLength('severity', 20)
            ->requirePresence('severity', 'create')
            ->notEmptyString('severity');

        $validator
            ->scalar('uptime')
            ->maxLength('uptime', 20)
            ->requirePresence('uptime', 'create')
            ->notEmptyString('uptime');

        $validator
            ->scalar('traptime')
            ->maxLength('traptime', 30)
            ->requirePresence('traptime', 'create')
            ->notEmptyString('traptime');

        $validator
            ->scalar('formatline')
            ->maxLength('formatline', 255)
            ->requirePresence('formatline', 'create')
            ->notEmptyString('formatline');

        $validator
            ->integer('trapread')
            ->notEmptyString('trapread');

        return $validator;
    }

    /**
     * @param int $id
     * @return bool
     */
    public function existsById($id) {
        return $this->exists(['Snmptt.id' => $id]);
    }

    /**
     * @param SnmpttListIndexFilter $SnmpttListIndexFilter
     * @param PaginateOMat|null $PaginateOMat
     * @return array
     */
    public function getSnmpttIndex(SnmpttListIndexFilter $SnmpttListIndexFilter, PaginateOMat $PaginateOMat = null) {
        $query = $this->find('all')
            ->where($SnmpttListIndexFilter->indexFilter())
            ->order($SnmpttListIndexFilter->getOrderForPaginator('Snmptt.id', 'desc'))
            ->disableHydration();

        if ($PaginateOMat === null) {
            //Just execute query
            if (empty($query)) {
                return [];
            }
            $result = $query->toArray();
        } else {
            if ($PaginateOMat->useScroll()) {
                $result = $this->scroll($query, $PaginateOMat->getHandler(), false);
            } else {
                $result = $this->paginate($query, $PaginateOMat->getHandler(), false);
            }
        }

        return $result;
    }
}
